/* --- [ usi_logger.c ] -------------------------------------------------------
*
* Datum             : 09.06.2008
* Compiler          : avr-gcc 4.3.0
*
* Description       : R232 Empfangs Umsetzer
*                     und protokolliert auf PORT_B zum Master.
*
****************************************************************************/

#include <avr/io.h>
#include <util/delay.h>
#include <avr/pgmspace.h>

const char  COPYR[] PROGMEM = { "RS-232-Logger V_1_02 "
                                __DATE__
                                " (c) ths"
                              };

#define  FOREVER    ( 1 == 1 )

#define  SEL_0      PD2
#define  SEL_1      PD3
#define  SEL_2      PD4
#define  STROBE     PD6
#define  P_ACK      PD5

/*----------------------------------------------------------*/
void uart_putc( char  ch )
/*=======================*/
   {
   // Warten solange bis Zeichen gesendet wurde
   //
   loop_until_bit_is_set( UCSRA, UDRE );

   // Ausgabe des Zeichens
   //
   UDR = ch;

   if ( ch == '\n' )
      {
      // Warten solange bis Zeichen gesendet wurde
      //
      loop_until_bit_is_set( UCSRA, UDRE );

      // Ausgabe des Zeichens
      //
      UDR = '\r';
      }
   }

/*----------------------------------------------------------*/
void uart_puts( const char *  s )
/*==============================*/
   {
   register char  ch;

    while ( ( ch = pgm_read_byte( s++ ) ) )
       {
		 uart_putc( ch );
  	    }
    }


//----------------------------------------------------------------------------
int main( void )
/*=============*/
   {
   unsigned char  sel = 0;

   // Strobe und LED auf Ausgang
   //
   DDRD |= _BV( STROBE );

   // ACK und SEL auf Eingang
   //
   DDRD &= ~( _BV( P_ACK ) | _BV( SEL_2 ) | _BV( SEL_1 ) | _BV( SEL_0 ) );

   // Strobe auf high, ACK PullUp ein
   //
   PORTD |= _BV( STROBE ) | _BV( P_ACK ) | _BV( SEL_2 ) | _BV( SEL_1 ) | _BV( SEL_0 );

   // setup for reading Config Byte
   //
   DDRB  = 0b00000000;
   PORTB = 0b00000000;

   _delay_ms( 5 );      // Reset + 64ms + 5ms, Master legt Pegel fuer 80ms an  

   sel = PINB;
   sel = PINB;
   sel = PINB;
   sel = PINB;
   sel = PINB;

   _delay_ms( 25 );     // Reset + 64ms + 5ms + 25ms = 94ms, 14 ms nach Master

   // setup for normal Mode
   //
   DDRB  = 0b11111111;
   PORTB = 0b00000000;

   // serielle Schnittstelle initialisieren
   //
   switch ( sel & 0x0F )     // Baudrate
      {
      case  0:  UBRRH = 0x23;  UBRRL = 0xFF;  break;      //     75
      case  1:  UBRRH = 0x11;  UBRRL = 0xFF;  break;      //    150
      case  2:  UBRRH = 0x08;  UBRRL = 0xFF;  break;      //    300
      case  3:  UBRRH = 0x04;  UBRRL = 0x7F;  break;      //    600
      case  4:  UBRRH = 0x02;  UBRRL = 0x3F;  break;      //   1200
      case  5:  UBRRH = 0x01;  UBRRL = 0x1F;  break;      //   2400
      case  6:  UBRRH = 0x00;  UBRRL = 0x8F;  break;      //   4800
      case  7:  UBRRH = 0x00;  UBRRL = 0x5F;  break;      //   7200
      case  8:  UBRRH = 0x00;  UBRRL = 0x47;  break;      //   9600
      case  9:  UBRRH = 0x00;  UBRRL = 0x2F;  break;      //  14400
      case 10:  UBRRH = 0x00;  UBRRL = 0x23;  break;      //  19200
      case 11:  UBRRH = 0x00;  UBRRL = 0x17;  break;      //  28800
      case 12:  UBRRH = 0x00;  UBRRL = 0x11;  break;      //  38400
      case 13:  UBRRH = 0x00;  UBRRL = 0x0B;  break;      //  57600
      case 14:  UBRRH = 0x00;  UBRRL = 0x08;  break;      //  76800
      case 15:  UBRRH = 0x00;  UBRRL = 0x05;  break;      // 115200
      }

   UCSRB |= ( 1<<TXEN )  | ( 1<<RXEN );

   switch ( sel & 0x30 )    // Zeichenlaenge
      {
      case  0x00:  UCSRC |= ( 0<<UCSZ1 ) | ( 0<<UCSZ0 );  break;      // 5 Bit
      case  0x10:  UCSRC |= ( 0<<UCSZ1 ) | ( 1<<UCSZ0 );  break;      // 6 Bit
      case  0x20:  UCSRC |= ( 1<<UCSZ1 ) | ( 0<<UCSZ0 );  break;      // 7 Bit
      case  0x30:  UCSRC |= ( 1<<UCSZ1 ) | ( 1<<UCSZ0 );  break;      // 8 Bit
      }

   uart_puts( COPYR );

   while ( FOREVER )
      {
      // testen, ob ein Byte empfangen
      //
      if ( bit_is_set( UCSRA, RXC ) )
         {
         // Datum auf Port legen
         //
         PORTB = UDR;

         // Strobe auf low
         //
         PORTD &= ~_BV( STROBE );

         // Warte auf ACK == low
         //
         loop_until_bit_is_clear( PIND, P_ACK ) ;

         // Strobe auf high
         //
         PORTD |= _BV( STROBE );

         // Warte auf ACK == high
         //
         loop_until_bit_is_set( PIND, P_ACK ) ;

         uart_putc( 0x00 );
         }

      }
   }



